#ifdef PETSC_RCS_HEADER
static char vcid[] = "$Id: ilubs.c,v 1.23 1999/01/12 23:15:29 bsmith Exp bsmith $";
#endif
/*
   Defines a ILU factorization preconditioner with BlockSolve95 interface.
   See notes about scaling and permuting at the top of the file iccbs.c.
*/
#include "petsc.h"

#if defined(HAVE_BLOCKSOLVE) && !defined(USE_PETSC_COMPLEX)
#include "src/pc/pcimpl.h"
#include "src/pc/impls/ilu/ilu.h"
#include "src/ksp/kspimpl.h"
#include "src/mat/impls/rowbs/mpi/mpirowbs.h"


extern int PCPreSolve_MPIRowbs(PC,KSP);
extern int PCPostSolve_MPIRowbs(PC,KSP);

EXTERN_C_BEGIN
#undef __FUNC__  
#define __FUNC__ "PCSetUp_ILU_MPIRowbs"
int PCSetUp_ILU_MPIRowbs(PC pc)
{
  PC_ILU       *ilu = (PC_ILU *) pc->data;
  MatStructure pflag;
  Mat          Amat, Pmat;
  int          ierr;

  PetscFunctionBegin;  
  ierr = PCGetOperators(pc,&Amat,&Pmat,&pflag); CHKERRQ(ierr);
  if (Amat != Pmat && Amat->type == MATMPIROWBS) {
    SETERRQ(PETSC_ERR_SUP,0,"Does not support different Amat and\n\
                 Pmat with MATMPIROWBS format for both.  Use a different format for\n\
                 Amat (e.g., MATMPIAIJ) and keep Pmat the same.");
  }
  if (Amat->type == MATMPIROWBS) {
    pc->presolve    = PCPreSolve_MPIRowbs;
    pc->postsolve   = PCPostSolve_MPIRowbs;
  }
  PetscFunctionReturn(0);
}
EXTERN_C_END

#else
#undef __FUNC__  
#define __FUNC__ "MatNull_MPIRowbs_ILU"
int MatNull_MPIRowbs_ILU(void)
{
  PetscFunctionBegin;  
  PetscFunctionReturn(0);
}
#endif



